@@ -17,14 +17,14 @@ class ApoJWT():
             Audience names are typically base address URLs. Ex: https://example.com
         algorithm: The algorithm to use when encoding/decoding. Default HS256
         admin_permission: Optional permission allowing full access to JWTs carrying this permission. USE CAREFULLY
-        * token_finder: Function used to retrieve the access JWT from http request Authorization header. Default None
 
     Framework Configuration
         async_framework: Tells ApoJWT to use async decorators instead of the normal (FastAPI needs this True). Default False
+        * token_finder: Function used to retrieve the access JWT from http request Authorization header. Default None
         * exception_handler: HTTP error handling function given an HTTP code and message as arguments
     """
 
-    def __init__(self, secret: str, exp_period: int=900, iss: str="", server_audience: list=[], algorithm: str="HS256", admin_permission: str=None, template: str=None, async_framework: bool=False, token_finder=None, exception_handler=None):        
+    def __init__(self, secret: str, exp_period: int=900, iss: str="", server_audience: list=[], algorithm: str="HS256", admin_permission: str=None, async_framework: bool=False, token_finder=None, exception_handler=None):        
         self.__exp_period = int(exp_period)
         self.__refresh_exp_period = 0
         self.__iss = str(iss)
@@ -36,18 +36,9 @@ class ApoJWT():
         self.__refresh_f = None
         self.__refresh_secret = None
 
-        if template == "flask":
-            from flask_template import create_flask_template
-            self.__async = False
-            self.__token_f, self.__exception_handler = create_flask_template()
-        elif template == "fastapi":
-            from fastapi_template import create_fastapi_template 
-            self.__async = True
-            self.__token_f, self.__exception_handler = create_fastapi_template()
-        else:
-            self.__async = async_framework
-            self.__token_f = token_finder
-            self.__exception_handler = exception_handler
+        self.__async = async_framework
+        self.__token_f = token_finder
+        self.__exception_handler = exception_handler
 
 
     def config_refresh(self, refresh_secret: str, refresh_exp_period: int=86400, refresh_finder=None):
@@ -77,9 +68,11 @@ class ApoJWT():
             @wraps(fn)
             async def wrapper(*args, **kwargs):
                 if self.__token_f is None:
-                    raise TypeError("ApoJWT requires the token_finder attribute to be defined for validating JWTs")
+                    raise ValueError("ApoJWT requires the token_finder attribute to be defined for validating JWTs")
                 try:
                     token = self.__token_f(*args, **kwargs)
+                    if token is None or token == "":
+                        raise Exception
                 except Exception as e:
                     if self.__exception_handler is None:
                         raise jwt.exceptions.InvalidTokenError(f"JWT is missing - Token Finder failed: {e}")
@@ -97,9 +90,11 @@ class ApoJWT():
             @wraps(fn)
             def wrapper(*args, **kwargs):
                 if self.__token_f is None:
-                    raise TypeError("ApoJWT requires the token_finder attribute to be defined for validating JWTs")
+                    raise ValueError("ApoJWT requires the token_finder attribute to be defined for validating JWTs")
                 try:
                     token = self.__token_f(*args, **kwargs)
+                    if token is None or token == "":
+                        raise Exception
                 except Exception as e:
                     if self.__exception_handler is None:
                         raise jwt.exceptions.InvalidTokenError(f"JWT is missing - Token Finder failed: {e}")
@@ -134,6 +129,8 @@ class ApoJWT():
                         raise TypeError("ApoJWT requires the token_finder attribute to be defined for validating JWTs")
                     try:
                         token = self.__token_f(*args, **kwargs)
+                        if token is None or token == "":
+                            raise Exception
                     except Exception as e:
                         if self.__exception_handler is None:
                             raise jwt.exceptions.InvalidTokenError(f"JWT is missing - Token Finder failed: {e}")
@@ -151,9 +148,9 @@ class ApoJWT():
                         
                     if permission not in decoded_permissions:
                         if self.__exception_handler is None:
-                            raise jwt.exceptions.InvalidTokenError(f"403: JWT is not authorized for this action")
+                            raise jwt.exceptions.InvalidTokenError(f"403: JWT does not have permission for this action")
                         else:
-                            kwarg_dict = dict(token_data=dict(), token_subject="", token_permissions=list(), access_token="", code=403, msg="JWT is not authorized for this action")
+                            kwarg_dict = dict(token_data=dict(), token_subject="", token_permissions=list(), access_token="", code=403, msg="JWT does not have permission for this action")
                             self.__form_kwargs(self.__exception_handler, kwarg_dict, kwargs)
                             self.__exception_handler(*args, **kwargs)
                             raise NotImplementedError("ApoJWT Exception Handler must not return control")
@@ -168,6 +165,8 @@ class ApoJWT():
                         raise TypeError("ApoJWT requires the token_finder attribute to be defined for validating JWTs")
                     try:
                         token = self.__token_f(*args, **kwargs)
+                        if token is None or token == "":
+                            raise Exception
                     except Exception as e:
                         if self.__exception_handler is None:
                             raise jwt.exceptions.InvalidTokenError(f"JWT is missing - Token Finder failed: {e}")
@@ -185,9 +184,9 @@ class ApoJWT():
 
                     if permission not in decoded_permissions:
                         if self.__exception_handler is None:
-                            raise jwt.exceptions.InvalidTokenError(f"403: JWT is not authorized for this action")
+                            raise jwt.exceptions.InvalidTokenError("JWT does not have permission for this action")
                         else:
-                            kwarg_dict = dict(token_data=dict(), token_subject="", token_permissions=list(), access_token="", code=403, msg="JWT is not authorized for this action", *args, **kwargs)
+                            kwarg_dict = dict(token_data=dict(), token_subject="", token_permissions=list(), access_token="", code=403, msg="JWT does not have permission for this action", *args, **kwargs)
                             self.__form_kwargs(self.__exception_handler, kwarg_dict, kwargs)
                             self.__exception_handler(*args, **kwargs)
                             raise NotImplementedError("ApoJWT Exception Handler must not return control")
@@ -209,6 +208,8 @@ class ApoJWT():
                     kwargs.pop("refresh_data")
                 try:
                     ref_token = self.__refresh_f(*args, **kwargs)
+                    if ref_token is None or ref_token == "":
+                        raise Exception
                 except Exception as e:
                     if self.__exception_handler is None:
                         raise jwt.exceptions.InvalidTokenError(f"Refresh JWT is missing - Refresh Token Finder failed: {e}")
@@ -231,6 +232,8 @@ class ApoJWT():
                     kwargs.pop("refresh_data")
                 try:
                     ref_token = self.__refresh_f(*args, **kwargs)
+                    if ref_token is None or ref_token == "":
+                        raise Exception
                 except Exception as e:
                     if self.__exception_handler is None:
                         raise jwt.exceptions.InvalidTokenError(f"Refresh JWT is missing - Refresh Token Finder failed: {e}")
@@ -308,9 +311,9 @@ class ApoJWT():
                 raise not_implemented
         except jwt.exceptions.ExpiredSignatureError:
             if self.__exception_handler is None:
-                raise jwt.exceptions.ExpiredSignatureError(f"403: JWT signature has expired")
+                raise jwt.exceptions.ExpiredSignatureError(f"401: JWT signature has expired")
             else:
-                kwarg_dict = dict(token_data=dict(), token_subject="", token_permissions=list(), access_token="", code=403, msg="JWT signature has expired")
+                kwarg_dict = dict(token_data=dict(), token_subject="", token_permissions=list(), access_token="", code=401, msg="JWT signature has expired")
                 self.__form_kwargs(self.__exception_handler, kwarg_dict, kwargs)
                 self.__exception_handler(*args, **kwargs)
                 raise not_implemented
@@ -330,6 +333,14 @@ class ApoJWT():
                 self.__form_kwargs(self.__exception_handler, kwarg_dict, kwargs)
                 self.__exception_handler(*args, **kwargs)
                 raise not_implemented
+        except jwt.exceptions.DecodeError as e:
+            if self.__exception_handler is None:
+                raise jwt.exceptions.InvalidTokenError(f"401: Bad Format - JWT could not be decoded")
+            else:
+                kwarg_dict = dict(token_data=dict(), token_subject="", token_permissions=list(), access_token="", code=401, msg=f"Bad Format - JWT could not be decoded: {e}")
+                self.__form_kwargs(self.__exception_handler, kwarg_dict, kwargs)
+                self.__exception_handler(*args, **kwargs)
+                raise not_implemented
         except jwt.exceptions.InvalidTokenError as e:
             if self.__exception_handler is None:
                 raise jwt.exceptions.InvalidTokenError(f"401: JWT is invalid")
@@ -340,6 +351,7 @@ class ApoJWT():
                 raise not_implemented
 
 
+
     def __form_kwargs(self, fn, new_kwargs: dict, kwargs):
         """Inspects the function 'fn' and decides which new_kwargs should be sent as keyword arguments"""
         fn_args = list(inspect.signature(fn).parameters.keys())
